@@ -2,6 +2,7 @@
 """CmdStanPy Module"""
 
 import atexit
+import logging
 import shutil
 import tempfile
 
@@ -21,25 +22,40 @@ _TMPDIR = tempfile.mkdtemp()
 _CMDSTAN_WARMUP = 1000
 _CMDSTAN_SAMPLING = 1000
 _CMDSTAN_THIN = 1
-_DOT_CMDSTANPY = '.cmdstanpy'
+_CMDSTAN_REFRESH = 100
 _DOT_CMDSTAN = '.cmdstan'
 
 
-def _cleanup_tmpdir():
+def _cleanup_tmpdir() -> None:
     """Force deletion of _TMPDIR."""
-    print('deleting tmpfiles dir: {}'.format(_TMPDIR))
+    logging.getLogger('cmdstanpy').info('deleting tmpfiles dir: %s', _TMPDIR)
     shutil.rmtree(_TMPDIR, ignore_errors=True)
-    print('done')
+    logging.getLogger('cmdstanpy').info('done')
 
 
 atexit.register(_cleanup_tmpdir)
 
 
 from ._version import __version__  # noqa
+from .install_cmdstan import rebuild_cmdstan
 from .model import CmdStanModel  # noqa
-from .stanfit import CmdStanGQ, CmdStanMCMC, CmdStanMLE, CmdStanVB  # noqa
+from .stanfit import (  # noqa
+    CmdStanGQ,
+    CmdStanMCMC,
+    CmdStanMLE,
+    CmdStanVB,
+    InferenceMetadata,
+    from_csv,
+)
 from .utils import set_cmdstan_path  # noqa
-from .utils import cmdstan_path, install_cmdstan, set_make_env
+from .utils import (
+    cmdstan_path,
+    cmdstan_version,
+    install_cmdstan,
+    set_make_env,
+    show_versions,
+    write_stan_json,
+)
 
 __all__ = [
     'set_cmdstan_path',
@@ -51,4 +67,10 @@ __all__ = [
     'CmdStanGQ',
     'CmdStanVB',
     'CmdStanModel',
+    'InferenceMetadata',
+    'from_csv',
+    'write_stan_json',
+    'show_versions',
+    'rebuild_cmdstan',
+    'cmdstan_version',
 ]
