@@ -9,10 +9,11 @@ import subprocess
 import sys
 from collections import OrderedDict
 from concurrent.futures import ThreadPoolExecutor
+from datetime import datetime
 from io import StringIO
 from multiprocessing import cpu_count
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Mapping, Optional, Union
+from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Union
 
 import ujson as json
 from tqdm.auto import tqdm
@@ -39,6 +40,7 @@ from cmdstanpy.utils import (
     MaybeDictToFilePath,
     SanitizedOrTmpFilePath,
     cmdstan_path,
+    cmdstan_version,
     cmdstan_version_before,
     do_command,
     get_logger,
@@ -47,6 +49,8 @@ from cmdstanpy.utils import (
 
 from . import progress as progbar
 
+OptionalPath = Union[str, os.PathLike, None]
+
 
 class CmdStanModel:
     # overview, omitted from doc comment in order to improve Sphinx docs.
@@ -89,13 +93,13 @@ class CmdStanModel:
     def __init__(
         self,
         model_name: Optional[str] = None,
-        stan_file: Optional[str] = None,
-        exe_file: Optional[str] = None,
+        stan_file: OptionalPath = None,
+        exe_file: OptionalPath = None,
         # TODO should be Literal['force'] not str
         compile: Union[bool, str] = True,
         stanc_options: Optional[Dict[str, Any]] = None,
         cpp_options: Optional[Dict[str, Any]] = None,
-        user_header: Optional[str] = None,
+        user_header: OptionalPath = None,
     ) -> None:
         """
         Initialize object given constructor args.
@@ -230,12 +234,12 @@ class CmdStanModel:
         return self._name
 
     @property
-    def stan_file(self) -> Optional[str]:
+    def stan_file(self) -> OptionalPath:
         """Full path to Stan program file."""
         return self._stan_file
 
     @property
-    def exe_file(self) -> Optional[str]:
+    def exe_file(self) -> OptionalPath:
         """Full path to Stan exe file."""
         return self._exe_file
 
@@ -251,7 +255,7 @@ class CmdStanModel:
             return result
         try:
             info = StringIO()
-            do_command(cmd=[self.exe_file, 'info'], fd_out=info)
+            do_command(cmd=[str(self.exe_file), 'info'], fd_out=info)
             lines = info.getvalue().split('\n')
             for line in lines:
                 kv_pair = [x.strip() for x in line.split('=')]
@@ -280,7 +284,7 @@ class CmdStanModel:
                 + self._compiler_options.compose_stanc()
                 + [
                     '--info',
-                    self.stan_file,
+                    str(self.stan_file),
                 ]
             )
             proc = subprocess.run(
@@ -297,6 +301,98 @@ class CmdStanModel:
             get_logger().debug(e)
             return result
 
+    def format(
+        self,
+        overwrite_file: bool = False,
+        canonicalize: Union[bool, str, Iterable[str]] = False,
+        max_line_length: int = 78,
+        *,
+        backup: bool = True,
+    ) -> None:
+        """
+        Run stanc's auto-formatter on the model code. Either saves directly
+        back to the file or prints for inspection
+
+
+        :param overwrite_file: If True, save the updated code to disk, rather
+            than printing it. By default False
+        :param canonicalize: Whether or not the compiler should 'canonicalize'
+            the Stan model, removing things like deprecated syntax. Default is
+            False. If True, all canonicalizations are run. If it is a list of
+            strings, those options are passed to stanc (new in Stan 2.29)
+        :param max_line_length: Set the wrapping point for the formatter. The
+            default value is 78, which wraps most lines by the 80th character.
+        :param backup: If True, create a stanfile.bak backup before
+            writing to the file. Only disable this if you're sure you have other
+            copies of the file or are using a version control system like Git.
+        """
+        if self.stan_file is None or not os.path.isfile(self.stan_file):
+            raise ValueError("No Stan file found for this module")
+        try:
+            cmd = (
+                [os.path.join(cmdstan_path(), 'bin', 'stanc' + EXTENSION)]
+                # handle include-paths, allow-undefined etc
+                + self._compiler_options.compose_stanc()
+                + [str(self.stan_file)]
+            )
+
+            if canonicalize:
+                if cmdstan_version_before(2, 29):
+                    if isinstance(canonicalize, bool):
+                        cmd.append('--print-canonical')
+                    else:
+                        raise ValueError(
+                            "Invalid arguments passed for current CmdStan"
+                            + " version({})\n".format(
+                                cmdstan_version() or "Unknown"
+                            )
+                            + "--canonicalize requires 2.29 or higher"
+                        )
+                else:
+                    if isinstance(canonicalize, str):
+                        cmd.append('--canonicalize=' + canonicalize)
+                    elif isinstance(canonicalize, Iterable):
+                        cmd.append('--canonicalize=' + ','.join(canonicalize))
+                    else:
+                        cmd.append('--print-canonical')
+
+            # before 2.29, having both --print-canonical
+            # and --auto-format printed twice
+            if not (cmdstan_version_before(2, 29) and canonicalize):
+                cmd.append('--auto-format')
+
+            if not cmdstan_version_before(2, 29):
+                cmd.append(f'--max-line-length={max_line_length}')
+            elif max_line_length != 78:
+                raise ValueError(
+                    "Invalid arguments passed for current CmdStan version"
+                    + " ({})\n".format(cmdstan_version() or "Unknown")
+                    + "--max-line-length requires 2.29 or higher"
+                )
+
+            out = subprocess.run(
+                cmd, capture_output=True, text=True, check=True
+            )
+            if out.stderr:
+                get_logger().warning(out.stderr)
+            result = out.stdout
+            if overwrite_file:
+                if result:
+                    if backup:
+                        shutil.copyfile(
+                            self.stan_file,
+                            str(self.stan_file)
+                            + '.bak-'
+                            + datetime.now().strftime("%Y%m%d%H%M%S"),
+                        )
+                    with (open(self.stan_file, 'w')) as file_handle:
+                        file_handle.write(result)
+            else:
+                print(result)
+
+        except (ValueError, RuntimeError) as e:
+            raise RuntimeError("Stanc formatting failed") from e
+
     @property
     def stanc_options(self) -> Dict[str, Union[bool, int, str]]:
         """Options to stanc compilers."""
@@ -332,7 +428,7 @@ class CmdStanModel:
         force: bool = False,
         stanc_options: Optional[Dict[str, Any]] = None,
         cpp_options: Optional[Dict[str, Any]] = None,
-        user_header: Optional[str] = None,
+        user_header: OptionalPath = None,
         override_options: bool = False,
     ) -> None:
         """
@@ -386,7 +482,7 @@ class CmdStanModel:
             src_time = os.path.getmtime(self._stan_file)
             exe_time = os.path.getmtime(exe_target)
             if exe_time > src_time and not force:
-                get_logger().info('found newer exe file, not recompiling')
+                get_logger().debug('found newer exe file, not recompiling')
                 if self._exe_file is None:  # called from constructor
                     self._exe_file = exe_target
                 return
@@ -475,10 +571,10 @@ class CmdStanModel:
 
     def optimize(
         self,
-        data: Union[Mapping[str, Any], str, None] = None,
+        data: Union[Mapping[str, Any], str, os.PathLike, None] = None,
         seed: Optional[int] = None,
-        inits: Union[Dict[str, float], float, str, None] = None,
-        output_dir: Optional[str] = None,
+        inits: Union[Dict[str, float], float, str, os.PathLike, None] = None,
+        output_dir: OptionalPath = None,
         sig_figs: Optional[int] = None,
         save_profile: bool = False,
         algorithm: Optional[str] = None,
@@ -639,7 +735,7 @@ class CmdStanModel:
     # pylint: disable=too-many-arguments
     def sample(
         self,
-        data: Union[Mapping[str, Any], str, None] = None,
+        data: Union[Mapping[str, Any], str, os.PathLike, None] = None,
         chains: Optional[int] = None,
         parallel_chains: Optional[int] = None,
         threads_per_chain: Optional[int] = None,
@@ -661,7 +757,7 @@ class CmdStanModel:
         adapt_metric_window: Optional[int] = None,
         adapt_step_size: Optional[int] = None,
         fixed_param: bool = False,
-        output_dir: Optional[str] = None,
+        output_dir: OptionalPath = None,
         sig_figs: Optional[int] = None,
         save_latent_dynamics: bool = False,
         save_profile: bool = False,
@@ -1056,7 +1152,7 @@ class CmdStanModel:
                 sys.stdout.write('\n')
                 get_logger().info('CmdStan done processing.')
 
-            get_logger().debug('runset\n%s', runset.__repr__())
+            get_logger().debug('runset\n%s', repr(runset))
 
             # hack needed to parse CSV files if model has no params
             # needed if exe is supplied without stan file
@@ -1081,22 +1177,32 @@ class CmdStanModel:
                     " force_one_process_per_chain to True"
                 )
 
+            errors = runset.get_err_msgs()
             if not runset._check_retcodes():
-                msg = 'Error during sampling:\n{}'.format(runset.get_err_msgs())
-                msg = '{}Command and output files:\n{}'.format(
-                    msg, runset.__repr__()
+                msg = (
+                    f'Error during sampling:\n{errors}\n'
+                    + f'Command and output files:\n{repr(runset)}\n'
+                    + 'Consider re-running with show_console=True if the above'
+                    + ' output is unclear!'
                 )
                 raise RuntimeError(msg)
+            if errors:
+                msg = (
+                    f'Non-fatal error during sampling:\n{errors}\n'
+                    + 'Consider re-running with show_console=True if the above'
+                    + ' output is unclear!'
+                )
+                get_logger().warning(msg)
 
             mcmc = CmdStanMCMC(runset)
         return mcmc
 
     def generate_quantities(
         self,
-        data: Union[Mapping[str, Any], str, None] = None,
+        data: Union[Mapping[str, Any], str, os.PathLike, None] = None,
         mcmc_sample: Union[CmdStanMCMC, List[str], None] = None,
         seed: Optional[int] = None,
-        gq_output_dir: Optional[str] = None,
+        gq_output_dir: OptionalPath = None,
         sig_figs: Optional[int] = None,
         show_console: bool = False,
         refresh: Optional[int] = None,
@@ -1223,12 +1329,13 @@ class CmdStanModel:
                         show_console=show_console,
                     )
 
-            if not runset._check_retcodes():
-                msg = 'Error during generate_quantities:\n{}'.format(
-                    runset.get_err_msgs()
-                )
-                msg = '{}Command and output files:\n{}'.format(
-                    msg, runset.__repr__()
+            errors = runset.get_err_msgs()
+            if errors:
+                msg = (
+                    f'Error during generate_quantities:\n{errors}\n'
+                    + f'Command and output files:\n{repr(runset)}\n'
+                    + 'Consider re-running with show_console=True if the above'
+                    + ' output is unclear!'
                 )
                 raise RuntimeError(msg)
             quantities = CmdStanGQ(runset=runset, mcmc_sample=mcmc_fit)
@@ -1236,10 +1343,10 @@ class CmdStanModel:
 
     def variational(
         self,
-        data: Union[Mapping[str, Any], str, None] = None,
+        data: Union[Mapping[str, Any], str, os.PathLike, None] = None,
         seed: Optional[int] = None,
         inits: Optional[float] = None,
-        output_dir: Optional[str] = None,
+        output_dir: OptionalPath = None,
         sig_figs: Optional[int] = None,
         save_latent_dynamics: bool = False,
         save_profile: bool = False,
@@ -1427,7 +1534,6 @@ class CmdStanModel:
         vb = CmdStanVB(runset)
         return vb
 
-    # pylint: disable=no-self-use
     def _run_cmdstan(
         self,
         runset: RunSet,
