@@ -68,7 +68,12 @@ def get_logger() -> logging.Logger:
         # add a default handler to the logger to INFO and higher
         handler = logging.StreamHandler()
         handler.setLevel(logging.INFO)
-        handler.setFormatter(logging.Formatter(logging.BASIC_FORMAT))
+        handler.setFormatter(
+            logging.Formatter(
+                '%(asctime)s - %(name)s - %(levelname)s - %(message)s',
+                "%H:%M:%S",
+            )
+        )
         logger.addHandler(handler)
     return logger
 
@@ -132,7 +137,6 @@ def get_latest_cmdstan(cmdstan_dir: str) -> Optional[str]:
     # unmunge as needed
     mmp = latest.split('.')
     if int(mmp[2]) < 0:
-        print("here")
         rc_num = str(int(mmp[2]) + 100)
         mmp[2] = "0-rc" + rc_num
         latest = '.'.join(mmp)
@@ -149,7 +153,10 @@ def validate_cmdstan_path(path: str) -> None:
         raise ValueError(f'No CmdStan directory, path {path} does not exist.')
     if not os.path.exists(os.path.join(path, 'bin', 'stanc' + EXTENSION)):
         raise ValueError(
-            'CmdStan installataion missing binaries, run "install_cmdstan"'
+            'CmdStan installataion missing binaries. '
+            'Re-install cmdstan by running command "install_cmdstan '
+            '--overwrite", or Python code "import cmdstanpy; '
+            'cmdstanpy.install_cmdstan(overwrite=True)"'
         )
 
 
@@ -173,20 +180,20 @@ def cmdstan_path() -> str:
     Validate, then return CmdStan directory path.
     """
     cmdstan = ''
-    if 'CMDSTAN' in os.environ:
+    if 'CMDSTAN' in os.environ and len(os.environ['CMDSTAN']) > 0:
         cmdstan = os.environ['CMDSTAN']
     else:
         cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
         if not os.path.exists(cmdstan_dir):
             raise ValueError(
-                'No CmdStan installation found, run "install_cmdstan" or'
-                ' (re)activate your conda environment!'
+                'No CmdStan installation found, run command "install_cmdstan"'
+                'or (re)activate your conda environment!'
             )
         latest_cmdstan = get_latest_cmdstan(cmdstan_dir)
         if latest_cmdstan is None:
             raise ValueError(
-                'No CmdStan installation found, run "install_cmdstan" or'
-                ' (re)activate your conda environment!'
+                'No CmdStan installation found, run command "install_cmdstan"'
+                'or (re)activate your conda environment!'
             )
         cmdstan = os.path.join(cmdstan_dir, latest_cmdstan)
         os.environ['CMDSTAN'] = cmdstan
@@ -292,7 +299,7 @@ def cxx_toolchain_path(
         if os.path.exists(os.path.join(toolchain_root, 'mingw64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
+                'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -316,7 +323,7 @@ def cxx_toolchain_path(
         elif os.path.exists(os.path.join(toolchain_root, 'mingw_64')):
             compiler_path = os.path.join(
                 toolchain_root,
-                'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
+                'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
                 'bin',
             )
             if os.path.exists(compiler_path):
@@ -368,7 +375,7 @@ def cxx_toolchain_path(
                 if version not in ('35', '3.5', '3'):
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
+                        'mingw64' if (sys.maxsize > 2**32) else 'mingw32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -393,7 +400,7 @@ def cxx_toolchain_path(
                 else:
                     compiler_path = os.path.join(
                         toolchain_root,
-                        'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
+                        'mingw_64' if (sys.maxsize > 2**32) else 'mingw_32',
                         'bin',
                     )
                     if os.path.exists(compiler_path):
@@ -650,7 +657,7 @@ def scan_sampler_csv(path: str, is_fixed_param: bool = False) -> Dict[str, Any]:
             if not is_fixed_param:
                 lineno = scan_warmup_iters(fd, dict, lineno)
                 lineno = scan_hmc_params(fd, dict, lineno)
-            lineno = scan_sampling_iters(fd, dict, lineno)
+            lineno = scan_sampling_iters(fd, dict, lineno, is_fixed_param)
         except ValueError as e:
             raise ValueError("Error in reading csv file: " + path) from e
     return dict
@@ -730,7 +737,7 @@ def scan_variational_csv(path: str) -> Dict[str, Any]:
             lineno += 1
         xs = line.split(',')
         variational_mean = [float(x) for x in xs]
-        dict['variational_mean'] = variational_mean
+        dict['variational_mean'] = np.array(variational_mean)
         dict['variational_sample'] = pd.read_csv(
             path,
             comment='#',
@@ -838,7 +845,7 @@ def parse_method_vars(names: Tuple[str, ...]) -> Dict[str, Tuple[int, ...]]:
     if names is None:
         raise ValueError('missing argument "names"')
     # note: method vars are currently all scalar so not checking for structure
-    return {v: tuple([k]) for (k, v) in enumerate(names) if v.endswith('__')}
+    return {v: (k,) for (k, v) in enumerate(names) if v.endswith('__')}
 
 
 def parse_stan_vars(
@@ -953,13 +960,21 @@ def scan_hmc_params(
 
 
 def scan_sampling_iters(
-    fd: TextIO, config_dict: Dict[str, Any], lineno: int
+    fd: TextIO, config_dict: Dict[str, Any], lineno: int, is_fixed_param: bool
 ) -> int:
     """
     Parse sampling iteration, save number of iterations to config_dict.
+    Also save number of divergences, max_treedepth hits
     """
     draws_found = 0
     num_cols = len(config_dict['column_names'])
+    if not is_fixed_param:
+        idx_divergent = config_dict['column_names'].index('divergent__')
+        idx_treedepth = config_dict['column_names'].index('treedepth__')
+        max_treedepth = config_dict['max_depth']
+        ct_divergences = 0
+        ct_max_treedepth = 0
+
     cur_pos = fd.tell()
     line = fd.readline().strip()
     while len(line) > 0 and not line.startswith('#'):
@@ -977,8 +992,16 @@ def scan_sampling_iters(
             )
         cur_pos = fd.tell()
         line = fd.readline().strip()
-    config_dict['draws_sampling'] = draws_found
+        if not is_fixed_param:
+            ct_divergences += int(data[idx_divergent])  # type: ignore
+            if int(data[idx_treedepth]) == max_treedepth:  # type: ignore
+                ct_max_treedepth += 1
+
     fd.seek(cur_pos)
+    config_dict['draws_sampling'] = draws_found
+    if not is_fixed_param:
+        config_dict['ct_divergences'] = ct_divergences
+        config_dict['ct_max_treedepth'] = ct_max_treedepth
     return lineno
 
 
@@ -1253,6 +1276,8 @@ def install_cmdstan(
     progress: bool = False,
     verbose: bool = False,
     cores: int = 1,
+    *,
+    interactive: bool = False,
 ) -> bool:
     """
     Download and install a CmdStan release from GitHub. Downloads the release
@@ -1260,7 +1285,7 @@ def install_cmdstan(
     to allow for transient network outages. Builds CmdStan executables
     and tests the compiler by building example model ``bernoulli.stan``.
 
-    :param version: CmdStan version string, e.g. "2.24.1".
+    :param version: CmdStan version string, e.g. "2.29.2".
         Defaults to latest CmdStan release.
 
     :param dir: Path to install directory.  Defaults to hidden directory
@@ -1284,36 +1309,60 @@ def install_cmdstan(
     :param cores: Integer, number of cores to use in the ``make`` command.
         Default is 1 core.
 
+    :param interactive: Boolean value; if true, ignore all other arguments
+        to this function and run in an interactive mode, prompting the user
+        to provide the other information manually through the standard input.
+
+        This flag should only be used in interactive environments,
+        e.g. on the command line.
+
     :return: Boolean value; ``True`` for success.
     """
     logger = get_logger()
-    args = {
-        "version": version,
-        "overwrite": overwrite,
-        "verbose": verbose,
-        "compiler": compiler,
-        "progress": progress,
-        "dir": dir,
-        "cores": cores,
-    }
-
     try:
-        from .install_cmdstan import main
+        from .install_cmdstan import (
+            InstallationSettings,
+            InteractiveSettings,
+            run_install,
+        )
 
-        main(args)
+        args: Union[InstallationSettings, InteractiveSettings]
+
+        if interactive:
+            if any(
+                [
+                    version,
+                    dir,
+                    overwrite,
+                    compiler,
+                    progress,
+                    verbose,
+                    cores != 1,
+                ]
+            ):
+                logger.warning(
+                    "Interactive installation requested but other arguments"
+                    " were used.\n\tThese values will be ignored!"
+                )
+            args = InteractiveSettings()
+        else:
+            args = InstallationSettings(
+                version=version,
+                overwrite=overwrite,
+                verbose=verbose,
+                compiler=compiler,
+                progress=progress,
+                dir=dir,
+                cores=cores,
+            )
+        run_install(args)
     # pylint: disable=broad-except
     except Exception as e:
-        logger.warning('CmdStan installation failed.')
-        logger.warning(str(e))
+        logger.warning('CmdStan installation failed.\n%s', str(e))
         return False
 
-    if dir is not None:
-        if version is not None:
-            set_cmdstan_path(os.path.join(dir, 'cmdstan-' + version))
-        else:
-            set_cmdstan_path(
-                os.path.join(dir, get_latest_cmdstan(dir))  # type: ignore
-            )
+    set_cmdstan_path(args.dir)
+
     return True
 
 
@@ -1382,7 +1431,9 @@ class MaybeDictToFilePath:
 
     def __init__(
         self,
-        *objs: Union[str, Mapping[str, Any], List[Any], int, float, None],
+        *objs: Union[
+            str, Mapping[str, Any], List[Any], int, float, os.PathLike, None
+        ],
     ):
         self._unlink = [False] * len(objs)
         self._paths: List[Any] = [''] * len(objs)
@@ -1397,7 +1448,7 @@ class MaybeDictToFilePath:
                 write_stan_json(data_file, obj)
                 self._paths[i] = data_file
                 self._unlink[i] = True
-            elif isinstance(obj, str):
+            elif isinstance(obj, (str, os.PathLike)):
                 if not os.path.exists(obj):
                     raise ValueError("File doesn't exist {}".format(obj))
                 self._paths[i] = obj
