@@ -1,29 +1,34 @@
 """CmdStanModel tests"""
 
+import contextlib
+import io
 import logging
 import os
 import shutil
 import tempfile
 import unittest
+from glob import glob
 from test import CustomTestCase
+from unittest.mock import MagicMock, patch
 
+import pytest
 from testfixtures import LogCapture, StringComparison
 
 from cmdstanpy.model import CmdStanModel
-from cmdstanpy.utils import EXTENSION
+from cmdstanpy.utils import EXTENSION, cmdstan_version_before
 
 HERE = os.path.dirname(os.path.abspath(__file__))
 DATAFILES_PATH = os.path.join(HERE, 'data')
 
 CODE = """data {
   int<lower=0> N;
-  int<lower=0,upper=1> y[N];
+  array[N] int<lower=0, upper=1> y;
 }
 parameters {
-  real<lower=0,upper=1> theta;
+  real<lower=0, upper=1> theta;
 }
 model {
-  theta ~ beta(1,1);  // uniform prior on interval 0,1
+  theta ~ beta(1, 1); // uniform prior on interval 0,1
   y ~ bernoulli(theta);
 }
 """
@@ -34,6 +39,7 @@ BERN_EXE = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
 BERN_BASENAME = 'bernoulli'
 
 
+# pylint: disable=too-many-public-methods
 class CmdStanModelTest(CustomTestCase):
     def test_model_good(self):
         # compile on instantiation, override model name
@@ -93,7 +99,6 @@ class CmdStanModelTest(CustomTestCase):
         model = CmdStanModel(stan_file=stan)
         self.assertTrue(model._fixed_param)
 
-    # pylint: disable=no-self-use
     def test_model_pedantic(self):
         stan_file = os.path.join(DATAFILES_PATH, 'bernoulli_pedantic.stan')
         with LogCapture(level=logging.WARNING) as log:
@@ -213,7 +218,6 @@ class CmdStanModelTest(CustomTestCase):
         self.assertTrue(os.path.exists(model.exe_file))
 
         info_dict = model.exe_info()
-        print(f'info={info_dict}')
         self.assertEqual(info_dict['STAN_THREADS'].lower(), 'false')
 
         more_opts = {'STAN_THREADS': 'TRUE'}
@@ -223,7 +227,6 @@ class CmdStanModelTest(CustomTestCase):
         self.assertTrue(os.path.exists(model.exe_file))
 
         info_dict2 = model.exe_info()
-        print(f'info2={info_dict2}')
         self.assertEqual(info_dict2['STAN_THREADS'].lower(), 'true')
 
         override_opts = {'STAN_NO_RANGE_CHECKS': 'TRUE'}
@@ -232,7 +235,6 @@ class CmdStanModelTest(CustomTestCase):
             force=True, cpp_options=override_opts, override_options=True
         )
         info_dict3 = model.exe_info()
-        print(f'info3={info_dict3}')
         self.assertEqual(info_dict3['STAN_THREADS'].lower(), 'false')
         # cmdstan#1056
         # self.assertEqual(info_dict3['STAN_NO_RANGE_CHECKS'].lower(), 'true')
@@ -307,7 +309,7 @@ class CmdStanModelTest(CustomTestCase):
 
     def test_repr(self):
         model = CmdStanModel(stan_file=BERN_STAN)
-        model_repr = model.__repr__()
+        model_repr = repr(model)
         self.assertIn('name=bernoulli', model_repr)
 
     def test_print(self):
@@ -374,6 +376,88 @@ class CmdStanModelTest(CustomTestCase):
         model2 = CmdStanModel(stan_file=stan)
         self.assertPathsEqual(model2.exe_file, exe)
 
+    @pytest.mark.skipif(
+        not cmdstan_version_before(2, 32),
+        reason="Deprecated syntax removed in Stan 2.32",
+    )
+    def test_model_format_deprecations(self):
+        stan = os.path.join(DATAFILES_PATH, 'format_me_deprecations.stan')
+
+        model = CmdStanModel(stan_file=stan, compile=False)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format()
+
+        formatted = sys_stdout.getvalue()
+        self.assertIn("//", formatted)
+        self.assertNotIn("#", formatted)
+        self.assertEqual(formatted.count('('), 5)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format(canonicalize=True)
+
+        formatted = sys_stdout.getvalue()
+        print(formatted)
+        self.assertNotIn("<-", formatted)
+        self.assertEqual(formatted.count('('), 0)
+
+        shutil.copy(stan, stan + '.testbak')
+        try:
+            model.format(overwrite_file=True, canonicalize=True)
+            self.assertEqual(len(glob(stan + '.bak-*')), 1)
+        finally:
+            shutil.copy(stan + '.testbak', stan)
+
+    @pytest.mark.skipif(
+        cmdstan_version_before(2, 29), reason='Options only available later'
+    )
+    def test_model_format_options(self):
+        stan = os.path.join(DATAFILES_PATH, 'format_me.stan')
+
+        model = CmdStanModel(stan_file=stan, compile=False)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format(max_line_length=10)
+        formatted = sys_stdout.getvalue()
+        self.assertGreater(len(formatted.splitlines()), 11)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format(canonicalize='braces')
+        formatted = sys_stdout.getvalue()
+        self.assertEqual(formatted.count('{'), 3)
+        self.assertEqual(formatted.count('('), 4)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format(canonicalize=['parentheses'])
+        formatted = sys_stdout.getvalue()
+        self.assertEqual(formatted.count('{'), 1)
+        self.assertEqual(formatted.count('('), 1)
+
+        sys_stdout = io.StringIO()
+        with contextlib.redirect_stdout(sys_stdout):
+            model.format(canonicalize=True)
+        formatted = sys_stdout.getvalue()
+        self.assertEqual(formatted.count('{'), 3)
+        self.assertEqual(formatted.count('('), 1)
+
+    @patch('cmdstanpy.utils.cmdstan_version', MagicMock(return_value=(2, 27)))
+    def test_format_old_version(self):
+        self.assertTrue(cmdstan_version_before(2, 28))
+
+        stan = os.path.join(DATAFILES_PATH, 'format_me.stan')
+        model = CmdStanModel(stan_file=stan, compile=False)
+        with self.assertRaisesRegexNested(RuntimeError, r"--canonicalize"):
+            model.format(canonicalize='braces')
+        with self.assertRaisesRegexNested(RuntimeError, r"--max-line"):
+            model.format(max_line_length=88)
+
+        model.format(canonicalize=True)
+
 
 if __name__ == '__main__':
     unittest.main()
